package require Tcl 8.5.7

namespace eval claim {
	
	########################
	# Script configuration #
	########################
	
	# prefix for public commands
	variable prefix			"!"
	# directory for storing script databases
	variable dirname		"./data/claim"
	# name of the database with claims
	variable claimdb		"claim.db"
	# name of the database with ignores
	variable ignoredb		"ignore.db"
	# minimum attendance for channel
	variable min_attendance	"10"
	# output messages speed
	# 1 - putserv
	# 2 - puthelp (*recommended*)
	# 3 - putquick
	# 4 - putdccraw (*NOT RECOMMENDED*)
	variable output_speed	"2"

	
	###################################################
	# STOP! STOP! STOP! STOP! STOP! STOP! STOP! STOP! #
	# STOP! STOP! STOP! STOP! STOP! STOP! STOP! STOP! #
	# STOP! STOP! STOP! STOP! STOP! STOP! STOP! STOP! #
	###################################################

	
	variable author			"Adium@RusNet <a.adium@gmail.com>"
	variable version		"2.0"
	variable date			"08-Dec-2009"
	variable unamespace		[namespace tail [namespace current]]
	
	variable chflag			$unamespace
	setudef flag			$chflag
	
	array set binds {
		"addclaim		$$	-|-"		"+ +claim"
		"delclaim		$$	o|o"		"- -claim"
		"editclaim		$$	o|o"		"+ + +edit +claimedit"
		"readclaim		$$	o|o"		"+ +read"
		"showclaim		$$	o|o"		"+ +show"
		"viewclaim		$$	o|o"		"+ +view"
		"clearclaim		$$	mn|m"		"+ -claimall"
		"addignore		$$	moyl|moyl"	"+badchan +bchan"
		"delignore		$$	moyl|moyl"	"-badchan -bchan"
		"editignore		$$	moyl|moyl"	"+editbadchan +ebadchan +ebchan"
		"readignore		$$	moyl|moyl"	"+badchans"
		"findignore		$$	moyl|moyl"	"+findbadchan +fbadchan +fbchan"
		"viewignore		$$	moyl|moyl"	"+viewbadchan +vbadchan +vbchan"
		"clearignore	$$	mo|m"		"-badchanall -allbadchans +clearbadchans +clearbchans +cbchans"
	}
	
	# create binds
	foreach key [array names binds] {
		# $key -- contains "proc_name$$bind_flags"
		# [lindex [string map [list "$$" " "] $key] 0] -- contains procedure name
		# [lindex [string map [list "$$" " "] $key] 1] -- contains bind flags for this procedure name

		foreach bind [split $binds($key)] {
			bind PUB [lindex [string map [list "$$" " "] $key] 1] ${prefix}$bind \
				[namespace current]::[lindex [string map [list "$$" " "] $key] 0]
		}
	}
	
	# procedures for "claim"
	proc addclaim {nick uhost hand chan text} {
		variable chflag; variable dirname; variable claimdb; variable ignoredb
		variable prefix; variable min_attendance
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set text [split $text]
		
		if {$text eq ""} {
			output $chan "$nick, \00314: \00304${prefix}+ \00314<\00304#_\00314> <\00304   \00314> <\00304    \(   \)\00314>\003"
			return 0
		}

		set channel [lindex $text 0]
		set attendance [lindex $text 1]
		set bot_services [lrange $text 2 end]
		
		if {[string range $channel 0 1] eq "<#" && [string index $channel end] eq ">"} {
			set channel [string range $channel 1 end-1]
		} elseif {[string index $channel 0] eq "<" && [string index $channel end] eq ">" \
				  && [string index $channel 1] ne "#"} {
			output $chan "$nick, \00304\002:\002 \00310      \00307#\.   \00307#$channel\00310,  \00307$channel\003"
			return 0
		} elseif {[string index $channel 0] ne "<" && [string index $channel end] ne ">" \
				  && [string index $channel 1] ne "#"} {
			output $chan "$nick, \00304\002:\002 \00310      \00307#\.   \00307#$channel\00310,  \00307$channel\003"
			return 0
		}
		
		if {$attendance eq ""} {
			output $chan "$nick, \00304\002:\002 \00310    \003"
			return 0
		}
		
		if {[string index $attendance 0] eq "<" && [string index $attendance end] eq ">" \
			&& [regexp -- {^\d+$} [string range $attendance 1 end-1]]} {
			set attendance [string range $attendance 1 end-1]
		} elseif {[string index $attendance 0] eq "<" && [string index $attendance end] eq ">" \
			&& ![regexp -- {^\d+$} [string range $attendance 1 end-1]]} {
			output $chan "$nick, \00304\002:\002 \00310    .    !\003"
			return 0
		} elseif {[string index $attendance 0] ne "<" && [string index $attendance end] ne ">" \
			&& ![regexp -- {^\d+$} [string range $attendance 1 end-1]]} {
			output $chan "$nick, \00304\002:\002 \00310    .    !\003"
			return 0
		}
		
		if {[expr {$attendance < $min_attendance}]} { 
			output $chan "$nick, \00304\002:\002 \00310      \00304$min_attendance\00310 .\003"
			return 0
		}

		if {$bot_services eq ""} {
			output $chan "$nick, \00304\002:\002 \00310      !\003"
			return 0
		}
		if {[string index $bot_services 0] eq "<" && [string index $bot_services end] eq ">"} {
			set bot_services [string range $bot_services 1 end-1]
		}

		set claimdb_data [readdata "$dirname/$claimdb"]
		foreach line $claimdb_data {
			if {[string tolower [lindex $line 1]] eq [string tolower $channel]} {
				output $chan "$nick, \00310   \00307\002$channel\002 \00310 .\003"
				return 0
			}
		}
	
		set ignoredb_data [readdata "$dirname/$ignoredb"]
		foreach line $ignoredb_data {
			if {[string tolower [lindex $line 1]] eq [string tolower $channel]} {
				output $chan "$nick, \00310   \00307\002$channel\002 \00310 .    .  : \00314[lrange $line 3 end]\003"
				return 0
			}
		}
	
		set newdata_string "[clock seconds] $channel $nick!$uhost $attendance $bot_services"
		set claimdb_data [linsert $claimdb_data end $newdata_string]

		#set line_count 0
		#foreach line $claimdb_data { incr line_count }
		writedata "$dirname/$claimdb" $data
		output $chan "$nick, \00310    $channel  .  \002\00304$num-\00310\002  .    \00307\002 \002 \00310 .\003"
	}
	
	
	proc delclaim {nick uhost hand chan text} {
		variable chflag; variable dirname
		variable claimdb; variable prefix
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set text [split $text]
		
		if {$text eq ""} {
			output $chan "$nick, \00314: \00304${prefix}- \00314<\00304\00314|\00304 \00314>\003"
			return 0
		}

		set claimdb_data [readdata "$dirname/$claimdb"]
		
		if {$claimdb_data eq ""} {
			output $chan "$nick, \00310     .\003"
			return
		}
		
		if {[regexp -- {^\d+$} $text] && ![regexp -- {[a-z-]} [string tolower $text]]} {
			if {[llength $claimdb_data] < $text} {
				output $chan "$nick, \00310   \00307\002$text\002 \00310 .    \002\00304[llength $claimdb_data]\00310\002 [end [llength $claimdb_data]].\003"
				return 0
			}
			set channame [lindex [lindex $claimdb_data [expr {$text - 1}]] 1]
			set claimdb_data [lreplace $claimdb_data [expr {$text - 1}] [expr {$text - 1}]]
			writedata "$dirname/$claimdb" $claimdb_data
			output $chan "$nick, \00310   \00307\002$text\002\00310   \00304$channame\00310 .\003"
		} elseif {[regexp -- {[a-z-]} [string tolower $text]]} {
			set num 0
			set j 0
			foreach line $claimdb_data {
				incr num
				if {[string tolower [lindex $line 1]] eq [string tolower $text]} {
					set j 1
					set claimdb_data [lreplace $claimdb_data [expr {$num - 1}] [expr {$num - 1}]]
					writedata "$dirname/$claimdb" $claimdb_data
					output $chan "$nick, \00310   \00304[lindex $line 1]\00304 \00310.\003"
					#set p [expr $p - 1]
					return 0
				}
			}
			
			if {$j eq "0"} {
				output $chan "$nick, \00310   \00304$text\00304 \00310 .\003"
			}
		}
	}
	
	
	proc editclaim {nick uhost hand chan text} {
		variable chflag; variable dirname; variable claimdb
		variable prefix; variable min_attendance
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set text [split $text]
		
		if {$text eq ""} {
			output $chan "$nick, \00314: \00304${prefix}+ \00314<\00304#_\00314> <\00304   \00314> <\00304    \(   \)\00314>\003"
			return 0
		}

		set channel [lindex $text 0]
		set attendance [lindex $text 1]
		set bot_services [lrange $text 2 end]
		
		if {[string range $channel 0 1] eq "<#" && [string index $channel end] eq ">"} {
			set channel [string range $channel 1 end-1]
		} elseif {[string index $channel 0] eq "<" && [string index $channel end] eq ">" \
				  && [string index $channel 1] ne "#"} {
			output $chan "$nick, \00304\002:\002 \00310      \00307#\.   \00307#$channel\00310,  \00307$channel\003"
			return 0
		} elseif {[string index $channel 0] ne "<" && [string index $channel end] ne ">" \
				  && [string index $channel 1] ne "#"} {
			output $chan "$nick, \00304\002:\002 \00310      \00307#\.   \00307#$channel\00310,  \00307$channel\003"
			return 0
		}
		
		set claimdb_data [readdata "$dirname/$claimdb"]
		set j 0; set num 0
		foreach line $claimdb_data {
			incr num
			if {[string tolower [lindex $line 1]] eq [string tolower $channel]} {
				set j 1
			}
		}
		
		if {$j eq "0"} {
			output $chan "$nick, \00310     \00304$channel\00310    .\003"

		}
	
		if {$attendance eq ""} {
			output $chan "$nick, \00304\002:\002 \00310    \003"
			return 0
		}
		
		if {[string index $attendance 0] eq "<" && [string index $attendance end] eq ">" \
			&& [regexp -- {^\d+$} [string range $attendance 1 end-1]]} {
			set attendance [string range $attendance 1 end-1]
		} elseif {[string index $attendance 0] eq "<" && [string index $attendance end] eq ">" \
			&& ![regexp -- {^\d+$} [string range $attendance 1 end-1]]} {
			output $chan "$nick, \00304\002:\002 \00310    .    !\003"
			return 0
		} elseif {[string index $attendance 0] ne "<" && [string index $attendance end] ne ">" \
			&& ![regexp -- {^\d+$} [string range $attendance 1 end-1]]} {
			output $chan "$nick, \00304\002:\002 \00310    .    !\003"
			return 0
		}
		
		if {[expr {$attendance < $min_attendance}]} { 
			output $chan "$nick, \00304\002:\002 \00310      \00304$min_attendance\00310 .\003"
			return 0
		}

		if {$bot_services eq ""} {
			output $chan "$nick, \00304\002:\002 \00310      !\003"
			return 0
		}
		if {[string index $bot_services 0] eq "<" && [string index $bot_services end] eq ">"} {
			set bot_services [string range $bot_services 1 end-1]
		}
		
		set newdata_string "[clock seconds] $channel $nick!$uhost $attendance $bot_services"
		set claimdb_data [lreplace $claimdb_data [expr {$num - 1}] [expr {$num - 1}] $newdata_string]
		writedata "$dirname/$claimdb" $data
		output $chan "$nick, \00310   \00304$channel\00310 .\003"
	}
	
	proc readclaim {nick uhost hand chan text} {
		variable chflag; variable dirname; variable claimdb
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set claimdb_data [readdata "$dirname/$claimdb"]

		if {$claimdb_data eq ""} { 
			output $chan "$nick, \00310     .\003"
			return 0
		}
	
		output $nick "\00306=====================\00307BOT CLAIM\0036=====================\003"
		
		set counter 0
		foreach line $claimdb_data {
			incr counter
			# $line has the structure of the form "seconds #chan nick!uhost attendance bot_services
			output $nick "\00303\[\00305$counter\00303\/[llength $claimdb_data]\]\00310 : \
					\00307[lindex $line 1] \00310 : \00304[lindex $line 2] \00310: \
					\00306[clock format [lindex $line 0] -format {%D (%A)}]  [clock format [lindex $line 0] -format {%H:%M:%S}] \
					\00310:\00307 [lindex $line 3] \00310 :\00314 [lrange $line 4 end]\003"
		}
		
		output $nick "\00306===================\00307END OF BOT CLAIM\0036==================\003"
	}

	proc showclaim {nick uhost hand chan text} {
		variable chflag; variable dirname
		variable claimdb; variable prefix
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set text [split $text]

		set nickname [lindex $text 0]
	
		if {$nickname eq ""} {
			output $chan "\00314: \00304${prefix}+ \00314<\00304#\00314> <\00304 \00314> <\00304   \00314>\003"
			return 0
		}

		set mylist [split [chanlist $chan]]
		set check [lsearch $mylist $nickname]
		
		if {$check eq "-1"} {
			output $chan "$nick, \00304\002:\002 \00310 \00304$nickname\00310    $chan.\003"
			return 0
		}
		output $chan "$nickname, \00314: \00304${prefix}+ \00314<\00304#\00314> <\00304 \00314> <\00304   \00314>\003"
	}
	
	proc viewclaim {nick uhost hand chan text} {
		variable chflag; variable dirname
		variable claimdb; variable prefix
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set text [split $text]
		
		if {$text eq ""} {
			output $chan "$nick, \00314: \00304${prefix}+ \00314<\00304 \00314>\003"
			return 0
		}
		
		set claim_num [lindex $text 0]
		if {[string index $claim_num 0] eq "<" && [string index $claim_num end] eq ">" \
			&& [regexp -- {^\d+$} [string range $claim_num 1 end-1]]} {
			set claim_num [string range $claim_num 1 end-1]
		} elseif {[string index $claim_num 0] eq "<" && [string index $claim_num end] eq ">" \
			&& ![regexp -- {^\d+$} [string range $claim_num 1 end-1]]} {
			output $chan "$nick, \00304\002:\002 \00310    .      !\003"
			return 0
		} elseif {[string index $claim_num 0] ne "<" && [string index $claim_num end] ne ">" \
			&& ![regexp -- {^\d+$} [string range $claim_num 1 end-1]]} {
			output $chan "$nick, \00304\002:\002 \00310    .      !\003"
			return 0
		}

		set claimdb_data [readdata "$dirname/$claimdb"]
		
		if {[expr {$claim_num > [llength $claimdb_data]}]} {
			output $chan "$nick, \00304\002:\002 \00310   [llength $claimdb_data] [end [llength $claimdb_data]]\003"
			return 0
		}
		
		if {$claimdb_data eq ""} { 
			output $chan "$nick, \00310     .\003"
			return 0
		}

		set counter 0; set j 0
		foreach line $claimdb_data {
			incr counter
			if {$counter eq $claim_num} {
				set j 1
				output $chan "\00303\[\00305$counter\00303\/[llength $claimdb_data]\]\00310 : \
					\00307[lindex $line 1] \00310 : \00304[lindex $line 2] \00310: \
					\00306[clock format [lindex $line 0] -format {%D (%A)}]  [clock format [lindex $line 0] -format {%H:%M:%S}] \
					\00310:\00307 [lindex $line 3] \00310 :\00314 [lrange $line 4 end]\003"
			}
		}
		if {$j eq "0"} {
			output $chan "$nick, \00310   \00304#$claim_num\00304 \00310 .\003"
		}
	}
	
	proc clearclaim {nick uhost hand chan text} {
		variable chflag; variable dirname; variable claimdb
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set claimdb_data [readdata "$dirname/$claimdb"]
	
		if {$claimdb_data eq ""} {
			output $chan "$nick, \00304\002:\002 \00310     .\003"
			return 0
		}
		
		set db_channels ""
		foreach line $claimdb_data {
			lappend db_channels "\00304[lindex $line 1]\003 "
		}
		writedata "$dirname/$claimdb" ""
		output $chan "$nick, \00310[end2 [llength $claimdb_data]] [end2 [llength $claimdb_data]] \
				\00304[llength $claimdb_data]\00310 [end [llength $claimdb_data]].    : $db_channels\003"
	}
	
	proc checkjoin {nick uhost hand chan} {
		variable chflag; variable dirname; variable claimdb
		
		if {![channel get $chan $chflag]} { return 0 }
			
		set claimdb_data [readdata "$dirname/$claimdb"]
		
		if {$claimdb_data eq ""} { return 0 }
		
		if {[matchattr $hand "o" $chan]} {
			if {[llength $claimdb_data] eq ""} { return 0 }
			if {$chan != $claim(jch) } { return }
			output $chan "$nick, \00310   :\002\00307 [llength $claimdb_data]\002\003" 
		}
	}

	bind MSG -|- +claimver	[namespace current]::version
	proc version {nick uhost hand text} {
		if {[string tolower $nick] eq "adium"} {
			set var "author version date unamespace"
			foreach p [split $var] {
				if {[info exist $p]} {
					output "PRIVMSG $nick :\00314Variable \$p = \00303$p\003"
				} else {
					output "PRIVMSG $nick :\00314Variable \$p \00304not found!\003"
				}
			}
			if {[info exist author] && [string tolower $author] != "Adium@RusNet <a.adium@gmail.com>"} {
				output "PRIVMSG $nick :\00314Script claim.tcl was changed! Now \$author = $author\003"
			}
		}
	}
	
	# procedures for "ignores"
	proc addignore {nick uhost hand chan text} {
		variable chflag; variable dirname
		variable ignoredb; variable prefix
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set text [split $text]
		
		if {$text eq ""} {
			output $chan "$nick, \00314: \00304${prefix}+badchan \00314<\00304#_\00314> <\00304 \00314>\003"
			return 0
		}

		set channel [lindex $text 0]
		set reason [lrange $text 1 end]
		
		if {[string range $channel 0 1] eq "<#" && [string index $channel end] eq ">"} {
			set channel [string range $channel 1 end-1]
		} elseif {[string index $channel 0] eq "<" && [string index $channel end] eq ">" \
				  && [string index $channel 1] ne "#"} {
			output $chan "$nick, \00304\002:\002 \00310      \00307#\.   \00307#$channel\00310,  \00307$channel\003"
			return 0
		} elseif {[string index $channel 0] ne "<" && [string index $channel end] ne ">" \
				  && [string index $channel 1] ne "#"} {
			output $chan "$nick, \00304\002:\002 \00310      \00307#\.   \00307#$channel\00310,  \00307$channel\003"
			return 0
		}
		
		if {$reason eq ""} {
			output $chan "$nick, \00304\002:\002 \00310         .\003"
			return 0
		}
		
		if {[string index $reason 0] eq "<" && [string index $reason end] eq ">"} {
			set reason [string range $reason 1 end-1]
		}

		set ignoredb_data [readdata "$dirname/$ignoredb"]
		foreach line $ignoredb {
			if {[string tolower [lindex $line 0]] eq [string tolower $channel]} {
				output $chan "$nick, \00310   \00307\002$channel\002 \00310 .\003"
				return 0
			}
		}
	
		set ignoredb_data [readdata "$dirname/$ignoredb"]
		foreach line $ignoredb_data {
			if {[string tolower [lindex $line 0]] eq [string tolower $channel]} {
				# TODO: add display author of ignore
				#scan [lindex $line 2] {%[^!]!%s} who who_host
				output $chan "$nick, \00310   \00307\002$channel\002 \00310 .\003"
				return 0
			}
		}
	
		set newdata_string "[clock seconds] $channel $nick!$uhost $reason"
		set ignoredb_data [linsert $ignoredb_data end $newdata_string]

		#set line_count 0
		#foreach line $ignoredb_data { incr line_count }
		writedata "$dirname/$ignoredb" $data
		output $chan "\00310   \00304$channel\00310  . \00314\[ \#\00307[llength $ignoredb_data]\00314\]\003"
	}
	
	proc delignore {nick uhost hand chan text} {
		variable chflag; variable dirname
		variable ignoredb; variable prefix
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set text [split $text]
		
		if {$text eq ""} {
			output $chan "$nick, \00314: \00304${prefix}-badchan \00314<\00304 \00314|\00304 \00314>\003"
			return 0
		}

		set ignoredb_data [readdata "$dirname/$ignoredb"]
		
		if {$ignoredb_data eq ""} {
			output $chan "$nick, \00310     .\003"
			return
		}
		
		if {[regexp -- {^\d+$} $text] && ![regexp -- {[a-z-]} [string tolower $text]]} {
			if {[llength $ignoredb_data] < $text} {
				output $chan "$nick, \00310   \00307\002$text\002 \00310 .    \002\00304[llength $ignoredb_data]\00310\002 [end3 [llength $ignoredb_data]].\003"
				return 0
			}
			set channame [lindex [lindex $ignoredb_data [expr {$text - 1}]] 1]
			set ignoredb_data [lreplace $ignoredb_data [expr {$text - 1}] [expr {$text - 1}]]
			writedata "$dirname/$ignoredb" $ignoredb_data
			output $chan "$nick, \00310   \00307\002$text\002\00310   \00304$channame\00310 .\003"
		} elseif {[regexp -- {[a-z-]} [string tolower $text]]} {
			set num 0
			set j 0
			foreach line $ignoredb_data {
				incr num
				if {[string tolower [lindex $line 1]] eq [string tolower $text]} {
					set j 1
					set ignoredb_data [lreplace $ignoredb_data [expr {$num - 1}] [expr {$num - 1}]]
					writedata "$dirname/$ignoredb" $ignoredb_data
					output $chan "$nick, \00310   \00304[lindex $line 1]\00304 \00310.\003"
					#set p [expr $p - 1]
					return 0
				}
			}
			
			if {$j eq "0"} {
				output $chan "$nick, \00310   \00304$text\00304 \00310 .\003"
			}
		}
	}

	proc editignore {nick uhost hand chan text} {
		variable chflag; variable dirname
		variable ignoredb; variable prefix
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set text [split $text]
		
		if {$text eq ""} {
			output $chan "$nick, \00314: \00304${prefix}+editbadchan \00314<\00304#_\00314> <\00304 \00314>\003"
			return 0
		}

		set channel [lindex $text 0]
		set reason [lrange $text 1 end]
		
		if {[string range $channel 0 1] eq "<#" && [string index $channel end] eq ">"} {
			set channel [string range $channel 1 end-1]
		} elseif {[string index $channel 0] eq "<" && [string index $channel end] eq ">" \
				  && [string index $channel 1] ne "#"} {
			output $chan "$nick, \00304\002:\002 \00310      \00307#\.   \00307#$channel\00310,  \00307$channel\003"
			return 0
		} elseif {[string index $channel 0] ne "<" && [string index $channel end] ne ">" \
				  && [string index $channel 1] ne "#"} {
			output $chan "$nick, \00304\002:\002 \00310      \00307#\.   \00307#$channel\00310,  \00307$channel\003"
			return 0
		}
		
		set ignoredb_data [readdata "$dirname/$ignoredb"]
		set j 0; set num 0
		foreach line $ignoredb_data {
			incr num
			if {[string tolower [lindex $line 1]] eq [string tolower $channel]} {
				set j 1
			}
		}
		
		if {$j eq "0"} {
			output $chan "$nick, \00310     \00304$channel\00310    .\003"

		}
	
		if {$reason eq ""} {
			output $chan "$nick, \00304\002:\002 \00310    .\003"
			return 0
		}
		
		if {[string index $reason 0] eq "<" && [string index $reason end] eq ">"} {
			set reason [string range $reason 1 end-1]
		}
		
		set newdata_string "[clock seconds] $channel $nick!$uhost $reason"
		set ignoredb_data [lreplace $ignoredb_data [expr {$num - 1}] [expr {$num - 1}] $newdata_string]
		writedata "$dirname/$ignoredb" $data
		output $chan "$nick, \00310   \00304$channel\00310 .\003"
	}

	proc readignore {nick uhost hand chan text} {
		variable chflag; variable dirname; variable ignoredb
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set ignoredb_data [readdata "$dirname/$ignoredb"]

		if {$ignoredb_data eq ""} { 
			output $chan "$nick, \00310     .\003"
			return 0
		}
		
		set text [split $text]
		set page [lindex $text 0]
		
		# it should not be here
		set maxlist 5
		
		if {$page eq ""} { set page 1 }
		
		if {[string index $page 0] eq "<" && [string index $page end] eq ">" \
			&& [regexp -- {^\d+$} [string range $page 1 end-1]]} {
			set page [string range $page 1 end-1]
		} elseif {[string index $page 0] eq "<" && [string index $page] eq ">" \
			&& ![regexp -- {^\d+$} [string range $page 1 end-1]]} {
			output $chan "$nick, \00304\002:\002 \00310   .\003"
			return 0
		} elseif {[string index $page 0] ne "<" && [string index $page end] ne ">" \
			&& ![regexp -- {^\d+$} [string range $page 1 end-1]]} {
			output $chan "$nick, \00304\002:\002 \00310   .\003"
			return 0
		}
		
		if {$page > [expr {[expr {[llength $ignoredb_data] / 5}] + 1}]} {
			output $chan "$nick, \00304\002:\002 \00310 $page  .\003"
			return 0
		}
		
		if {[expr {[expr {[llength $ignoredb_data] / 5}] + 1}] ne "0"} {
			set tmp [expr {[expr {$page - 1}] * $maxlist}]
			output $nick "\00306===\00307CHAN-IGNORES: PAGE \00303$page\/[expr {[expr {[llength $ignoredb_data] / 5}] + 1}]\0036===\003"
			for {set i [expr {[expr {$page - 1}] * $maxlist]}} {$i < [expr {[expr {$page * $maxlist}] - 1}]} {incr i} {
				incr tmp
				set line [lindex $ignoredb_data $i]
				# $line has the structure of the form "seconds #chan nick!uhost ignore_reason
				output $nick "\00303\[\00305$tmp\00303\/[llength $ignoredb_data]\]\00310 : \00307[lindex $line 1] \
							  \00310 : \00304[lindex $line 2] \00310: [clock format [lindex $line 0] -format {%D (%A)}] \
							   [clock format [lindex $line 0] -format {%H:%M:%S}] \00310:\00314 [lrange $line 3 end]\003"
			}
			output $nick "\00306===\00307END OF PAGE \00303$page\/[expr {[expr {[llength $ignoredb_data] / 5}] + 1}]\00306===\003"
		} else {
			set counter 0
			output $nick "\00306===\00307CHAN-IGNORES: PAGE \00303$page\/[expr {[expr {[llength $ignoredb_data] / 5}] + 1}]\0036===\003"
			foreach line $ignoredb_data {
				incr counter
				# $line has the structure of the form "seconds #chan nick!uhost ignore_reason
				output $nick "\00303\[\00305$tmp\00303\/[llength $ignoredb_data]\]\00310 : \00307[lindex $line 1] \
							  \00310 : \00304[lindex $line 2] \00310: [clock format [lindex $line 0] -format {%D (%A)}] \
							   [clock format [lindex $line 0] -format {%H:%M:%S}] \00310:\00314 [lrange $line 3 end]\003"
			}
			output $nick "\00306===\00307END OF PAGE \00303$page\/[expr {[expr {[llength $ignoredb_data] / 5}] + 1}]\00306===\003"
		}
	}

	proc findignore { nick uhost hand chan text } {
		variable chflag; variable dirname
		variable ignoredb; variable prefix
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set text [split $text]
		set channel [lindex $text 0]
		
		if {$channel eq ""} {
			output $chan "$nick, \00314: \00304${prefix}+fbadchan \00314<\00304  \00314>\003"
			return 0
		}
		
		
		if {[string index $channel 0] eq "#"} {
			set channel [string range $channel 1 end]
		}
		if {[string index $channel 0] eq "*"} {
			set channel [string range $channel 1 end]
		}
		if {[string index $channel end] eq "*"} {
			set channel [string range $channel 0 end-1]
		}
		
		set ignoredb_data [readdata "$dirname/$ignoredb"]
		
		set j 0
		foreach line $ignoredb_data {
			if {[string match "*[string tolower $channel]*" [string tolower [lindex $line 1]]]} {
				incr j
			}
		}
		if {$j > 0} {
			set result ""
			foreach line $ignoredb_data {
				if {[string match "*[string tolower $channel]*" [string tolower [lindex $line 1]]]} {
					lappend result [lindex $line 1]
				}
			}
			output $chan "$nick, \00310    \00304$j\00310 [end3 $j]: $result"
			output $chan "$nick, \00310     ,  \00304${prefix}+viewbadchan \00314<\00304#\00314>\003"
		}
		if {$j eq "0"} {
			output $chan "$nick, \00310  \00304$channel \00310  .\003"
			return 0
		}
	}
	
	proc viewignore {nick uhost hand chan text} {
		variable chflag; variable dirname
		variable ignoredb; variable prefix
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set text [split $text]
		
		if {$text eq ""} {
			output $chan "$nick, \00314: \00304${prefix}+viewbchan \00314<\00304#\00314>\003"
			return 0
		}
		
		set channel [lindex $text 0]
		if {[string range $channel 0 1] eq "<#" && [string index $channel end] eq ">"} {
			set channel [string range $channel 1 end-1]
		} elseif {[string index $channel 0] eq "<" && [string index $channel end] eq ">" \
				  && [string index $channel 1] ne "#"} {
			output $chan "$nick, \00304\002:\002 \00310      \00307#\.   \00307#$channel\00310,  \00307$channel\003"
			return 0
		} elseif {[string index $channel 0] ne "<" && [string index $channel end] ne ">" \
				  && [string index $channel 1] ne "#"} {
			output $chan "$nick, \00304\002:\002 \00310      \00307#\.   \00307#$channel\00310,  \00307$channel\003"
			return 0
		}

		set ignoredb_data [readdata "$dirname/$ignoredb"]
		
		if {$ignoredb_data eq ""} { 
			output $chan "$nick, \00310     .\003"
			return 0
		}

		set counter 0; set j 0
		foreach line $ignoredb_data {
			incr counter
			if {[string tolower $channel] eq [string tolower [lindex $line 1]]} {
				set j 1
				output $nick "\00303\[\00305$counter\00303\/[llength $ignoredb_data]\]\00310 : \00307[lindex $line 1] \
							  \00310 : \00304[lindex $line 2] \00310: [clock format [lindex $line 0] -format {%D (%A)}] \
							   [clock format [lindex $line 0] -format {%H:%M:%S}] \00310:\00314 [lrange $line 3 end]\003"
			}
		}
		if {$j eq "0"} {
			output $chan "$nick, \00310   \00304$channel\00304 \00310 .\003"
		}
	}

	proc clearignore {nick uhost hand chan text} {
		variable chflag; variable dirname; variable ignoredb
		
		if {![channel get $chan $chflag]} { return 0 }
		
		init_databases
		
		set ignoredb_data [readdata "$dirname/$ignoredb"]
	
		if {$ignoredb_data eq ""} {
			output $chan "$nick, \00304\002:\002 \00310     .\003"
			return 0
		}
		
		writedata "$dirname/$ignoredb" ""
		output $chan "$nick, \00310[end2 [llength $ignoredb_data]] [end2 [llength $claimdb_data]] \
				\00304[llength $claimdb_data]\00310 [end3 [llength $claimdb_data]].\003"
	}
	
	proc readdata {file} {
		if {![file exists $file]} {
			return ""
		} else {
			if {[catch {set fileio [open $file r]} r]} {return $r}
			set lines ""
			while {![eof $fileio]} {
				set line [gets $fileio]
				if {![string is space $line]} {lappend lines $line}
			}
			close $fileio
			return $lines
		}
	}
	
	proc writedata {file data} {
		if {[catch {set fileio [open "$file" "w+"]} r]} {return $r}
		foreach line [string range $data 0 end] {
			puts $fileio [join $line]
		}
		flush $fileio
		close $fileio
	}
	
	proc output {target str} {
		variable output_speed
		
		if {[string index $target 0] eq "#" && ![channel get $target usecolors]} {
				set str [stripcodes ruagcb $str]
		}
		
		switch -- $output_speed {
			"1"			{ putserv "PRIVMSG $target :$str" }
			"2"			{ puthelp "PRIVMSG $target :$str" }
			"3"			{ putquick "PRIVMSG $target :$str" }
			"4"			{ set msg "PRIVMSG $target :[string range $str 0 end]\n"
						  putdccraw 0 [string length $msg] $msg
						}
			default		{ puthelp "PRIVMSG $target :$str" }
		}
	}
	
	proc end {t} {
		if {$t eq "11" || $t eq "12" || $t eq "13" || $t eq "14"} {return ""}
		switch -- [string index $t end] { 1 { return "" } 2 - 3 - 4 { return "" } default { return "" } }
	}
	
	proc end2 {t} {
		if {$t eq "1"} {return ""} else {return ""}
	}
	
	proc end3 {t} {
		if {$t eq "11" || $t eq "12" || $t eq "13" || $t eq "14"} {return ""}
		switch -- [string index $t end] { 1 { return "" } 2 - 3 - 4 { return "" } default { return "" } }
	}
	
	proc init_databases {} {
		variable dirname; variable claimdb; variable ignoredb
		if {![file exists $dirname]} { file mkdir $dirname }
		if {![file exists "$dirname/$claimdb"]} { writedata "$dirname/$claimdb" "" }
		if {![file exists "$dirname/$ignoredb"]} { writedata "$dirname/$ignoredb" "" }
	}
	
	putlog "[namespace tail [namespace current]].tcl by $author \[version $version\] \[$date\] successfully loaded"

}
